@@ -9,42 +9,47 @@ Command-line entry point
 # w0703 catch Exception
 # r1705 else after return
 # pylint: disable=c0111,w1202,r1705
+# pylint: disable=logging-fstring-interpolation, fixme, import-outside-toplevel
 
 
 # DON'T import logger globally here
 # we need to be able to mess inside the logger module
 # before it gets loaded from another way
 
+from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
+import logging
 import asyncio
 
-from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
 
-from pkg_resources import resource_string
+from pkg_resources import resource_string #, resource_exists, resource_filename
 
 from asynciojobs import Scheduler, Job
 
-import logging
 from asyncssh.logging import set_log_level as asyncssh_set_log_level
 
-from rhubarbe.config import Config
-from rhubarbe.imagesrepo import ImagesRepo
-from rhubarbe.selector import (Selector,
-                               add_selector_arguments, selected_selector)
-from rhubarbe.action import Action
-from rhubarbe.display import Display
-from rhubarbe.display_curses import DisplayCurses
-from rhubarbe.node import Node
-from rhubarbe.imageloader import ImageLoader
-from rhubarbe.imagesaver import ImageSaver
-from rhubarbe.monitor.nodes import MonitorNodes
-from rhubarbe.monitor.loop import MonitorLoop
-from rhubarbe.monitor.phones import MonitorPhones
-from rhubarbe.monitor.leases import MonitorLeases
-from rhubarbe.monitor.accountsmanager import AccountsManager
-from rhubarbe.ssh import SshProxy
-from rhubarbe.leases import Leases
-from rhubarbe.inventory import Inventory
-from rhubarbe.inventoryphones import InventoryPhones
+# apparently using relative imports is - at least that's what pylint says
+# defining global variables like 'config'
+# pylint: disable = redefined-outer-name
+from .config import Config
+from .imagesrepo import ImagesRepo
+from .selector import Selector, add_selector_arguments, selected_selector
+from .action import Action
+from .display import Display
+from .display_curses import DisplayCurses
+from .node import Node
+from .imageloader import ImageLoader
+from .imagesaver import ImageSaver
+from .monitor.loop import MonitorLoop
+from .monitor.nodes import MonitorNodes
+from .monitor.phones import MonitorPhones
+from .monitor.pdus import MonitorPdus
+from .monitor.leases import MonitorLeases
+from .monitor.accountsmanager import AccountsManager
+from .ssh import SshProxy
+from .leases import Leases
+from .inventory import Inventory
+from .inventoryphones import InventoryPhones
+from .inventorypdus import InventoryPdus
 
 
 # a supported command comes with a driver function
@@ -703,6 +708,46 @@ def monitorphones(*argv):
 ####################
 
 
+@subcommand
+def monitorpdus(*argv):
+
+    # xxx hacky - do a side effect in the logger module
+    import rhubarbe.logger
+    rhubarbe.logger.logger = rhubarbe.logger.monitor_logger
+    from rhubarbe.logger import logger
+
+    usage = """
+    Cyclic probe all known pdus, and reports real-time status
+    at a sidecar service over websockets
+    """
+    config = Config()
+    parser = ArgumentParser(usage=usage,
+                            formatter_class=ArgumentDefaultsHelpFormatter)
+    parser.add_argument(
+        "-c", "--cycle",
+        default=config.value('monitor', 'cycle_pdus'),
+        type=float,
+        help="Delay to wait between 2 probes of each pdu")
+    parser.add_argument(
+        "-u", "--sidecar-url", dest="sidecar_url",
+        default=Config().value('sidecar', 'url'),
+        help="url for the sidecar server")
+    parser.add_argument(
+        "-v", "--verbose", action='store_true')
+    parser.add_argument("names", nargs='*', help="optionally provide device names")
+    args = parser.parse_args(argv)
+
+    logger.info("Using all pdus")
+    monitorpdus = MonitorPdus(**vars(args))
+
+    MonitorLoop("monitorpdus").run(
+        monitorpdus.run_forever(),
+        logger)
+    return 0
+
+####################
+
+
 @subcommand
 def monitorleases(*argv):
 
@@ -716,7 +761,7 @@ def monitorleases(*argv):
     the sidecar channel, which triggers leases acquisition right away.
     See config for defaults.
     """
-    config = Config()
+    #config = Config()
     parser = ArgumentParser(
         usage=usage, formatter_class=ArgumentDefaultsHelpFormatter)
     parser.add_argument(
@@ -826,10 +871,90 @@ def template(*argv):
         selected = getattr(args, nodes_or_phones)
         if selected:
             show_template(nodes_or_phones)
+    return 0
 
 ####################
 
 
+@subcommand
+def pdu(*argv):
+    usage = """
+    manage PDUs; examples:
+
+        rhubarbe-pdu list               # lists the known PDUs and devices by name
+                                        # all forms of list show STATIC INFO only
+                                        # (no remote device involved)
+        rhubarbe-pdu list r2lab         # here r2lab is the name of a pdu_host
+                                        # here again it's a static info
+        rhubarbe-pdu list jaguar        # works with devices too
+
+
+        rhubarbe-pdu status r2lab       # this time the PDU is probed
+                                        # for its current status
+        rhubarbe-pdu status jaguar      # works with devices too
+
+
+        rhubarbe-pdu on jaguar
+        rhubarbe-pdu off panther
+        rhubarbe-pdu reset n300
+
+    Note that for the OFF and RESET variants, there is an attempt to perform
+    a soft shutdown first (i.e. to contact the device by ssh and to run
+    `shutdown -h now` in that context)
+    for that the device needs to be properly configured
+    """
+    parser = ArgumentParser(usage=usage,
+                            formatter_class=ArgumentDefaultsHelpFormatter)
+    # not yet used
+    #parser.add_argument("-v", "--verbose", default=False, action='store_true')
+    parser.add_argument("command", help="one of the supported contrib commands")
+    parser.add_argument("extras", nargs="*", help="additional args are passed as is")
+    args = parser.parse_args(argv)
+
+    command, extras = args.command, args.extras
+
+    def die():
+        parser.print_help()
+        exit(255)
+
+    # export env variables picked in config
+    # and uppercased as is the tradition for global env. variables
+
+    try:
+        inventory_pdus = InventoryPdus.load()   # pylint: disable=no-member
+
+        match command:
+            case 'list':
+                inventory_pdus.list(extras)
+
+            case 'on' | 'off' | 'status' | 'reset':
+                match extras:
+                    case (name, ):
+                        # status accepts a pdu_host name
+                        if command == 'status':
+                            try:
+                                pdu_host = inventory_pdus.get_pdu_host(name)
+                                retcod = asyncio.run(pdu_host.probe())
+                                exit(retcod)
+                            except ValueError:
+                                pass
+                        # otherwise a device name is expected
+                        try:
+                            device = inventory_pdus.get_device(name)
+                            retcod = asyncio.run(device.run_action(command))
+                            exit(retcod)
+                        except ValueError as exc:
+                            print(exc)
+                            exit(255)
+            case _:
+                print(f"unknown command {command}")
+                die()
+    except ValueError as exc:
+        print(exc)
+        exit(255)
+####################
+
+
 @subcommand
 def version(*_):
     from rhubarbe.version import __version__
